@@ -3,23 +3,22 @@ from collections import defaultdict
 from concurrent.futures import Future, ProcessPoolExecutor, as_completed
 from pathlib import Path
 from typing import Dict, Iterator, List, Set, Tuple, Union
-import itertools
 
 import networkx as nx
 import pandas as pd
 from tqdm import tqdm
 
 from pyscomotif.constants import (AMINO_ACID_ALPHABET,
-                                    AMINO_ACID_RELAXED_GROUPS_MAP,
-                                    INDEX_ANGLE_BIN_SIZE,
-                                    INDEX_DISTANCE_BIN_SIZE)
+                                  AMINO_ACID_RELAXED_GROUPS_MAP,
+                                  INDEX_ANGLE_BIN_SIZE,
+                                  INDEX_DISTANCE_BIN_SIZE)
 from pyscomotif.data_containers import Residue, Residue_pair_data
 from pyscomotif.index_folders_and_files import index_folder_exists
 from pyscomotif.residue_data_dicts import extract_residue_data
 from pyscomotif.utils import (
     angle_between_two_vectors,
-    detect_the_compression_algorithm_used_in_the_index, get_bin_number,
-    get_sorted_2_tuple, pairwise_euclidean_distance,
+    detect_the_compression_algorithm_used_in_the_index, flatten_iterable,
+    get_bin_number, get_sorted_2_tuple, pairwise_euclidean_distance,
     read_compressed_and_pickled_file, sort_and_join_2_strings)
 
 
@@ -317,9 +316,9 @@ def update_map_of_PDBs_with_all_residue_pairs(
             del map_of_PDBs_with_all_residue_pairs[PDB_ID]
             del visited_nodes_map[PDB_ID]
             continue
-        
+
         map_of_PDBs_with_all_residue_pairs[PDB_ID].extend(connectivity_checked_residue_pairs) # Update the list of residue pairs of the PDB
-        visited_nodes_map[PDB_ID].update(set(itertools.chain.from_iterable(connectivity_checked_residue_pairs)))
+        visited_nodes_map[PDB_ID].update(set(flatten_iterable(connectivity_checked_residue_pairs)))
 
     return
 
@@ -339,7 +338,7 @@ def find_PDBs_with_all_residue_pairs(
         concurrent_executor.submit(get_PDBs_that_contain_the_residue_pair, pair_of_full_residue_IDs, residue_pair_data, distance_delta_thr, angle_delta_thr, index_folder_path, compression)
         for pair_of_full_residue_IDs, residue_pair_data in all_pairs_of_residues_to_check.items()
     ]
-
+    
     map_of_PDBs_with_all_residue_pairs: Dict[str, List[Tuple[str,str]]]
     map_of_PDBs_with_all_residue_pairs_initialized_flag: bool = False
     visited_nodes_map: Dict[str, Set[str]] # Used for connectivity check
@@ -356,7 +355,7 @@ def find_PDBs_with_all_residue_pairs(
             map_of_PDBs_with_all_residue_pairs = defaultdict(list, PDBs_that_contain_the_residue_pair)
             map_of_PDBs_with_all_residue_pairs_initialized_flag = True
             visited_nodes_map = {
-                PDB_ID:set(itertools.chain.from_iterable(list_of_residue_pairs))
+                PDB_ID:set(flatten_iterable(list_of_residue_pairs))
                 for PDB_ID, list_of_residue_pairs in map_of_PDBs_with_all_residue_pairs.items()
             }
 
@@ -382,6 +381,11 @@ def run_subgraph_monomorphism(motif_MST: nx.Graph, pairs_of_residues: List[Tuple
     candidate_PDB_graph = nx.Graph(pairs_of_residues)
     add_resname_as_node_attribute(candidate_PDB_graph)
 
+    # NOTE: We don't need to use the edge_matcher functionality of the GraphMatcher because in the previous step we performed a connectivity check,
+    # which ensures that each residue pair of the candidate PDB is in the correct geometrical arrangement AND connected to the correct residue pair. 
+    # If the connectivity check had not been performed, candidate PDBs with residue pairs in the correct geometrical arrangement BUT connected in
+    # the wrong way would pass the monomorphism check given it does not check edge values, only the residue types. This kind of edge case can for example happen
+    # when an epitope has a chain of repeating residue pairs (i.e (S,T),(T,S),(S,T), ...).
     graph_matcher = nx.algorithms.isomorphism.GraphMatcher(
         G1 = candidate_PDB_graph, G2 = motif_MST, 
         node_match = lambda node1,node2: node1['resname'] == node2['resname'] # Match based on residue name
@@ -427,7 +431,7 @@ def filter_out_PDBs_with_unconnected_residue_pairs(
 
         PDB_ID = submited_futures[future]
         monomorphism_checked_motifs = future.result()
-        if not monomorphism_checked_motifs: # These are the false positive PDBs, i.e PDBs that have all the residue pairs but where the monomorphism check doesn't find any similar motif as a result of unconnected pairs of residues 
+        if not monomorphism_checked_motifs: # These are the false positive PDBs, i.e PDBs that have all the residue pairs but where the monomorphism check doesn't find any similar motif 
             continue
 
         filtered_PDBs_with_all_residue_pairs[PDB_ID] = monomorphism_checked_motifs
@@ -451,7 +455,7 @@ def solve_motif_MST(
             map_of_PDBs_with_all_residue_pairs = defaultdict(list, PDBs_that_contain_the_residue_pair)
             map_of_PDBs_with_all_residue_pairs_initialized_flag = True
             visited_nodes_map = {
-                PDB_ID:set(itertools.chain.from_iterable(list_of_residue_pairs))
+                PDB_ID:set(flatten_iterable(list_of_residue_pairs))
                 for PDB_ID, list_of_residue_pairs in map_of_PDBs_with_all_residue_pairs.items()
             }
 
@@ -463,7 +467,7 @@ def solve_motif_MST(
             )
 
     # Non-parallel version of filter_out_PDBs_with_unconnected_residue_pairs. Using while loop + popitem()
-    # instead of a for loop because it reduces the RAM usage.
+    # instead of a for loop because it reduces RAM usage.
     add_resname_as_node_attribute(motif_MST) # Needed for subgraph monomorphism, see run_subgraph_monomorphism().
     filtered_PDBs_with_all_residue_pairs: Dict[str, List[nx.Graph]] =  {}
     while map_of_PDBs_with_all_residue_pairs:
